@@ -10,7 +10,13 @@ from traitlets import List, Unicode
 
 from empack.file_packager import pack_python_core
 
-from jupyterlite.constants import SHARE_LABEXTENSIONS, UTF8
+from jupyterlite.constants import (
+    SHARE_LABEXTENSIONS,
+    LAB_EXTENSIONS,
+    JUPYTERLITE_JSON,
+    UTF8,
+    FEDERATED_EXTENSIONS,
+)
 from jupyterlite.addons.federated_extensions import (
     FederatedExtensionAddon,
     ENV_EXTENSIONS,
@@ -123,6 +129,7 @@ class XeusPythonEnv(FederatedExtensionAddon):
             outname=Path(self.cwd.name) / "python_data",
             version=PYTHON_VERSION,
             export_name="globalThis.Module",
+            download_emsdk="latest",
         )
 
         # Find the federated extensions in the emscripten-env and install them
@@ -158,6 +165,17 @@ class XeusPythonEnv(FederatedExtensionAddon):
                 ],
             )
 
+        jupyterlite_json = manager.output_dir / JUPYTERLITE_JSON
+        lab_extensions_root = manager.output_dir / LAB_EXTENSIONS
+        lab_extensions = self.env_extensions(lab_extensions_root)
+
+        yield dict(
+            name="patch:xeus",
+            doc=f"ensure {JUPYTERLITE_JSON} includes the federated_extensions",
+            file_dep=[*lab_extensions, jupyterlite_json],
+            actions=[(self.patch_jupyterlite_json, [jupyterlite_json])],
+        )
+
     def create_env(self):
         """Create the xeus-python emscripten-32 env with either mamba, micromamba or conda."""
         if MAMBA_PYTHON_AVAILABLE:
@@ -252,6 +270,21 @@ class XeusPythonEnv(FederatedExtensionAddon):
             actions=[(self.copy_one, [pkg_path, dest])],
         )
 
+    def dedupe_federated_extensions(self, config):
+        if FEDERATED_EXTENSIONS not in config:
+            return
+
+        named = {}
+
+        # Making sure to dedupe extensions by keeping the most recent ones
+        for ext in config[FEDERATED_EXTENSIONS]:
+            if os.path.exists(self.output_extensions / ext["name"] / ext["load"]):
+                named[ext["name"]] = ext
+
+        config[FEDERATED_EXTENSIONS] = sorted(named.values(), key=lambda x: x["name"])
+
+        print("--- CONFIG AFTER DEDUPE", config)
+
     def __del__(self):
         # Cleanup
         shutil.rmtree(self.root_prefix, ignore_errors=True)
